/** *************************************************************
 * Proyecto CoMVeT                                             *
 * Proyecto de fin de carrera de Daniel Hctor Stolfi Rosso    *
 * Escuela Tcnica Superior de Ingeniera Informtica          *
 * Universidad de Mlaga - 2010                                *
 *                                                             *
 * Conexion.cpp                                                *
 * Clase abstracta que gestiona las conexiones (implementacin)*
 * Se crear una instancia de una de sus clases hijas.         *
 *                                                             *
 ************************************************************** */
// ---------------------------------------------------------------------------

#pragma hdrstop

#include <vcl.h>
#include <Math.hpp>
#pragma hdrstop
#include "edkErrorCode.h"
#include "EmoStateDLL.h"
#include "Conexion.h"
// ---------------------------------------------------------------------------

#pragma package(smart_init)

TConexion::TConexion() {
   // Reserva espacios de memoria
   evento = EE_EmoEngineEventCreate();
	estado = EE_EmoStateCreate();
   // Inicializa atributos
   cognitivo = COG_NEUTRAL;
   expresivo = EXP_NEUTRAL;
   potenciaCognitivo = 0.0;
   potenciaExpresivo = 0.0;
   cargaBat = 0;
   maximaBat = 0;
   wifi = NO_SIGNAL;
   tiempo = 0.0;
   for (unsigned int i = 0; i < NUM_SENALES_ELECTRODOS; i++) {
      electrodos[i] = EEG_CQ_NO_SIGNAL;
   }
}

TConexion::~TConexion() {
   // Libera la memoria reservada
   EE_EmoStateFree(estado);
   EE_EmoEngineEventFree(evento);
}

bool TConexion::desconectar() {
   // Desactiva la optimizacin
   EE_OptimizationDisable();
   EE_OptimizationParamFree(optimiz);
   // Desconecta
   error = EE_EngineDisconnect();
   return (error == EDK_OK);
}

bool TConexion::inicializar() {
   // Activa la optimizacin de las acciones utilizadas
   optimiz = EE_OptimizationParamCreate();
   // Modo cognitivo
   error = EE_OptimizationSetVitalAlgorithm(optimiz, EE_COGNITIV, COG_NEUTRAL | RETROCEDER);
   if (error == EDK_OK) {
      // Modo expresivo
      error = EE_OptimizationSetVitalAlgorithm(optimiz, EE_EXPRESSIV, EXP_NEUTRAL | AVANZAR);
      if (error == EDK_OK) {
         // Activa optimizacin
         error = EE_OptimizationEnable(optimiz);
      }
   }
   return (error == EDK_OK);
}

bool TConexion::eventoSiguiente() {
   // Obtiene el siguiente evento
   error = EE_EngineGetNextEvent(evento);
   if (error == EDK_OK) {
      // Actualiza el tipo
      tipo_evento = EE_EmoEngineEventGetType(evento);
      // Actualiza el valor de error
      error = EE_EmoEngineEventGetUserId(evento, &usuario);
   }
   return (error == EDK_OK);
}

bool TConexion::estadoActualizado() {
   if (tipo_evento == EE_EmoStateUpdated) {
      // Si se ha actualizado el estado se procede a actualizar los valores
      error = EE_EmoEngineEventGetEmoState(evento, estado);
      if (error == EDK_OK) {
         cognitivo = ES_CognitivGetCurrentAction(estado);
         potenciaCognitivo = ES_CognitivGetCurrentActionPower(estado);
         expresivo = ES_ExpressivGetLowerFaceAction(estado);
         potenciaExpresivo = ES_ExpressivGetLowerFaceActionPower(estado);
         tiempo = ES_GetTimeFromStart(estado);
         wifi = ES_GetWirelessSignalStatus(estado);
         ES_GetBatteryChargeLevel(estado, &cargaBat, &maximaBat);
         ES_GetContactQualityFromAllChannels(estado, electrodos, NUM_SENALES_ELECTRODOS);
      }
      return(error == EDK_OK);
   }
   else return false;
}

bool TConexion::dispositivoConectado() {
   // Devuelve true si el ltimo evento es EE_UserAdded
   return (tipo_evento == EE_UserAdded);
}

bool TConexion::dispositivoDesconectado() {
   // Devuelve true si el ltimo evento es EE_UserRemoved
   return(tipo_evento == EE_UserRemoved);
}

void TConexion::accion(AC_Accion_t* ac, unsigned int* potencia) {
   // Prioriza la deteccin del modo cognitivo
   if (cognitivo == RETROCEDER) {
      *ac = AC_Atras;
      // Adapta el valor de potencia al del programa
      *potencia = RoundTo(potenciaCognitivo * 10.0, 0);
   }
   else if (expresivo == AVANZAR) {
      *ac = AC_Adelante;
      // Adapta el valor de potencia al del programa
      *potencia = RoundTo(potenciaExpresivo * 10.0, 0);
   }
   else if (cognitivo == COG_NEUTRAL) {
      *ac = AC_Centro;
      *potencia = 0;
   }
   else if (expresivo == EXP_NEUTRAL) {
      *ac = AC_Centro;
      *potencia = 0;
   }
   else {
      // No se tienen en cuenta el resto de detecciones
      *ac = AC_Ninguna;
      *potencia = 0;
   }
}

SN_Senal_t TConexion::nivelSenal() {
   // Convierte el enumerado del API al del programa
   switch(wifi) {
      case NO_SIGNAL:   return SN_No;
      case BAD_SIGNAL:  return SN_Mala;
      case GOOD_SIGNAL: return SN_Buena;
   default:
      return SN_Ninguna;
   }
}

float TConexion::tiempoConexion() {
   return tiempo;
}

void TConexion::cargaBateria(int* carga, int* maxima) {
   *carga = cargaBat;
   *maxima = maximaBat;
}

void TConexion::estadoElectrodos(int* contactos, bool* ok) {
   // Calcula si el estado es apto para procesarlo
   *ok = true;
   unsigned int i = 0;
   while ((i < NUM_SENALES_ELECTRODOS) && *ok) {
      *ok = ((electrodos[i] != EEG_CQ_NO_SIGNAL) &&
             (electrodos[i] != EEG_CQ_VERY_BAD));
      i++;
   }
   // Reordena y copia en array de salida
   contactos[0] = electrodos[EE_CHAN_AF3]; // EE_CHAN_FP1 = EE_CHAN_AF3
   contactos[1] = electrodos[EE_CHAN_AF4]; // EE_CHAN_FP2 = EE_CHAN_AF4
   contactos[2] = electrodos[EE_CHAN_F3];
   contactos[3] = electrodos[EE_CHAN_F4];
   contactos[4] = electrodos[EE_CHAN_F7];
   contactos[5] = electrodos[EE_CHAN_F8];
   contactos[6] = electrodos[EE_CHAN_FC5];
   contactos[7] = electrodos[EE_CHAN_FC6];
   contactos[8] = electrodos[EE_CHAN_T7];
   contactos[9] = electrodos[EE_CHAN_T8];
   contactos[10] = electrodos[EE_CHAN_CMS]; // EE_CHAN_CMS = EE_CHAN_DRL
   contactos[11] = electrodos[EE_CHAN_DRL]; // EE_CHAN_DRL = EE_CHAN_CMS
   contactos[12] = electrodos[EE_CHAN_P7];
   contactos[13] = electrodos[EE_CHAN_P8];
   contactos[14] = electrodos[EE_CHAN_O1];
   contactos[15] = electrodos[EE_CHAN_O2];
}

float TConexion::nivelEntrenamiento() {
   // Devuelve el nivel de entrenamiento del usuario
   float nivel;
   if (EE_CognitivGetOverallSkillRating(usuario, &nivel) == EDK_OK) return nivel;
   else return 0;
}

UnicodeString TConexion::cadenaEstado() {
   // Pasa a cadena la informacin en el ltimo estado
   return FormatFloat("0.000", tiempo) +
          " Sen:" + FormatFloat("0", wifi) +
          " Bat:" + FormatFloat("0", cargaBat) + "/" + FormatFloat("0", maximaBat) +
          // Modo cognitivo
          " Acc:" + FormatFloat("0000", cognitivo) + " (" + FormatFloat("0.00", potenciaCognitivo) + ")" +
          // Modo expresivo
          " Exp:" + FormatFloat("0000", expresivo) + " (" + FormatFloat("0.00", potenciaExpresivo) + ")" +
          // Acelermetro
          " Gir:" + FormatFloat("0", giroX);

}

UnicodeString TConexion::cadenaPerfil() {
   // Devuelve en una cadena toda la informacin del perfil cargado
   unsigned long acciones;
   float nivel;
   int act1, act2, act3, act4;
   UnicodeString cad = "";

   cad = cad + "Firmas Cognitivas Entrenadas: ";
   if (EE_CognitivGetTrainedSignatureActions(usuario, &acciones) == EDK_OK) {
      cad = cad + cadenaBitsCognitivos(acciones);
   }

   cad = cad + "\nAcciones Cognitivas Activas: ";
   if (EE_CognitivGetActiveActions(usuario, &acciones) == EDK_OK) {
      cad = cad + cadenaBitsCognitivos(acciones);
   }

   if (EE_CognitivGetActionSensitivity(usuario, &act1, &act2, &act3, &act4)
      == EDK_OK) {
      cad = cad + "\nSesibilidad Retroceso: " + FormatFloat("0", act1);
   }

   if (EE_CognitivGetOverallSkillRating(usuario, &nivel) == EDK_OK) {
      cad = cad + "\nNivel de Entrenamiento Cognitivo: " + FormatFloat
         ("0.00%", nivel * 100);
   }

   cad = cad + "\nFirmas Expresivas Entrenadas: ";
   if (EE_ExpressivGetTrainedSignatureActions(usuario, &acciones) == EDK_OK) {
      cad = cad + cadenaBitsExpresivos(acciones);
   }

   if (EE_ExpressivGetThreshold(usuario, AVANZAR, EXP_SENSITIVITY, &act1)
      == EDK_OK) {
      cad = cad + "\nSensibilidad Avance: " + FormatFloat("0", act1);
   }

   return cad;
}

bool TConexion::cargaPerfil(char* fichero) {
   // Carga el perfil de usuario desde el disco
   error = EE_LoadUserProfile(usuario, fichero);
   return (error == EDK_OK);
}

bool TConexion::setSensibilidadAvance(unsigned int nivel) {
   // Modifica en el perfil de usuario la accin expresiva
   error = EE_ExpressivSetThreshold(usuario, AVANZAR, EXP_SENSITIVITY, nivel);
   return (error == EDK_OK);
}

unsigned int TConexion::getSensibilidadAvance() {
   // Obtiene el valor para la accin expresiva desde el perfil
   int valor = 0;
   error = EE_ExpressivGetThreshold(usuario, AVANZAR, EXP_SENSITIVITY, &valor);
   return Min(abs(valor), 1000);
}

bool TConexion::setSensibilidadRetroceso(unsigned int nivel) {
   // Modifica en el perfil de usuario la accin cognitiva
   int act1, act2, act3, act4;
   // Recupera los cuatro valores actuales
   error = EE_CognitivGetActionSensitivity(usuario, &act1, &act2, &act3, &act4);
   if (error == EDK_OK) {
      // Slo modifica el primero
      error = EE_CognitivSetActionSensitivity(usuario, nivel, act2, act3, act4);
   }
   return(error == EDK_OK);
}

unsigned int TConexion::getSensibilidadRetroceso() {
   // Obtiene el valor para la accin cognitiva desde el perfil
   int act1 = 0;
   int act2, act3, act4;
   // Utiliza slo el valor de la primera de las acciones
   error = EE_CognitivGetActionSensitivity(usuario, &act1, &act2, &act3, &act4);
   return Max(Min(abs(act1), 9), 1);
}

void TConexion::giro(GI_Giro_t* gi) {
   *gi = GI_Ninguno; // valor por defecto
   int anteX = giroX;
   if (error == EDK_OK) {
      int giroY;
      // Obtiene los valores
      error = EE_HeadsetGetGyroDelta(usuario, &giroX, &giroY);
      if (error == EDK_OK) {
         // Determina la accin segn el diferencial de giro
         if (abs(giroX) > GIRO_UMBRAL) {
            if ((giroX > 0) && (anteX < giroX)) *gi = GI_Derecha;
            else if ((giroX < 0) && (anteX > giroX)) *gi = GI_Izquierda;
         }
      }
      else giroX = 0;
   }
}

unsigned int TConexion::aceleracionGiro() {
   // Valor del giro ponderado segn el giro mximo
   return Min((int)abs(RoundTo((giroX * 10.0 / GIRO_MAXIMO), 0)), 10);
}

bool TConexion::inicializaGiro() {
   // Inicializa el giro
   error == EE_HeadsetGyroRezero(usuario);
   return(EDK_OK == error);
}

UnicodeString TConexion::tipoEstado() {
   switch(tipo_evento) {
   case EE_UnknownEvent:
      return "EE_UnknownEvent";
   case EE_EmulatorError:
      return "EE_EmulatorError";
   case EE_ReservedEvent:
      return "EE_ReservedEvent";
   case EE_UserAdded:
      return "EE_UserAdded";
   case EE_UserRemoved:
      return "EE_UserRemoved";
   case EE_EmoStateUpdated:
      return "EE_EmoStateUpdated";
   case EE_ProfileEvent:
      return "EE_ProfileEvent";
   case EE_CognitivEvent:
      return "EE_CognitivEvent";
   case EE_ExpressivEvent:
      return "EE_ExpressivEvent";
   case EE_InternalStateChanged:
      return "EE_InternalStateChanged";
   case EE_AllEvent:
      return "EE_AllEvent";
   default:
      return "----";
   }
}

UnicodeString TConexion::cadenaOptimizacion() {
   UnicodeString cad = "";
   if (EE_OptimizationIsEnabled) {
      unsigned int vector;
      // Optimizaciones cognitivas
      error = EE_OptimizationGetVitalAlgorithm(optimiz, EE_COGNITIV, &vector);
      if (error == EDK_OK) {
         cad = cadenaBitsCognitivos(vector);
         // Optimizaciones expresivas
         error = EE_OptimizationGetVitalAlgorithm(optimiz, EE_EXPRESSIV, &vector);
         if (error == EDK_OK) {
            cad = cad + " " + cadenaBitsExpresivos(vector);
         }
      }
   }
   return cad;
}

UnicodeString TConexion::cadenaError() {
   UnicodeString cad = "";
   switch(error) {
   case EDK_OK:
      cad = "Sin errores";
      break;
   case EDK_UNKNOWN_ERROR:
      cad = "Error interno";
      break;
   case EDK_INVALID_PROFILE_ARCHIVE:
      cad = "Perfil de usuario no vlido";
      break;
   case EDK_NO_USER_FOR_BASEPROFILE:
		cad = "Perfil no asociado con un usuario";
      break;
   case EDK_CANNOT_ACQUIRE_DATA:
      cad = "No es posible obtener datos de EEG";
      break;
   case EDK_BUFFER_TOO_SMALL:
      cad = "El almacn suministrado no tiene suficiente espacio";
      break;
   case EDK_OUT_OF_RANGE:
      cad = "El parmetro est fuera de rango";
      break;
   case EDK_INVALID_PARAMETER:
      cad = "Uno de los parmetros no es vlido";
      break;
   case EDK_PARAMETER_LOCKED:
      cad = "El parmetro est bloqueado por una deteccin y no puede modificarse en este momento";
      break;
   case EDK_COG_INVALID_TRAINING_ACTION:
      cad = "La accin a entrenar no es vlida";
      break;
   case EDK_COG_INVALID_TRAINING_CONTROL:
      cad = "El control a entrenar no es vlido";
      break;
   case EDK_COG_INVALID_ACTIVE_ACTION:
      cad = "Uno de los campos del vector no es vlido";
      break;
   case EDK_COG_EXCESS_MAX_ACTIONS:
      cad = "El vector contiene ms tipos de acciones que las permitidas";
      break;
   case EDK_EXP_NO_SIG_AVAILABLE:
      cad = "La firma entrenada no se encuenta disponible para su uso";
      break;
   case EDK_FILESYSTEM_ERROR:
      cad = "Error del sistema de ficheros";
      break;
   case EDK_INVALID_USER_ID:
      cad = "Identificador de usuario no vlido";
      break;
   case EDK_EMOENGINE_UNINITIALIZED:
      cad = "El motor Emotiv no se encuentra inicializado";
      break;
   case EDK_EMOENGINE_DISCONNECTED:
      cad = "Se ha perdido la conexin remota con el motor Emotiv";
      break;
   case EDK_EMOENGINE_PROXY_ERROR:
      cad = "No se puede realizar la conexin remota";
      break;
   case EDK_NO_EVENT:
      cad = "No hay nuevos eventos en este momento";
      break;
   case EDK_GYRO_NOT_CALIBRATED:
      cad = "Girscopo no calibrado";
      break;
   case EDK_OPTIMIZATION_IS_ON:
      cad = "La operacin ha fallado debido a la optimizacin";
      break;
   case EDK_RESERVED1:
      cad = "Error reservado";
      break;
   default:
      cad = "Error no catalogado";
   }
   return FormatFloat("0000", error) + ":" + cad;
}

UnicodeString TConexion::cadenaBitsCognitivos(unsigned long vector) {
   UnicodeString cad = "";
   if (vector & COG_NEUTRAL) {
      cad = cad + "COG.NEUTRAL ";
   }
   if (vector & COG_PUSH) {
      cad = cad + "EMPUJAR ";
   }
   if (vector & COG_PULL) {
      cad = cad + "TIRAR ";
   }
   if (vector & COG_LIFT) {
      cad = cad + "ARRIBA ";
   }
   if (vector & COG_DROP) {
      cad = cad + "ABAJO ";
   }
   if (vector & COG_LEFT) {
      cad = cad + "IZQUIERDA ";
   }
   if (vector & COG_RIGHT) {
      cad = cad + "DERECHA ";
   }
   if (vector & COG_ROTATE_LEFT) {
      cad = cad + "ROT.IZQUIERDA ";
   }
   if (vector & COG_ROTATE_RIGHT) {
      cad = cad + "ROT.DERECHA ";
   }
   if (vector & COG_ROTATE_CLOCKWISE) {
      cad = cad + "ROT.SETNIDO RELOJ ";
   }
   if (vector & COG_ROTATE_COUNTER_CLOCKWISE) {
      cad = cad + "ROT.SENTIDO CONTRARELOJ ";
   }
   if (vector & COG_ROTATE_FORWARDS) {
      cad = cad + "ROT.DIRECTA ";
   }
   if (vector & COG_ROTATE_REVERSE) {
      cad = cad + "ROT.INVERSA ";
   }
   if (vector & COG_DISAPPEAR) {
      cad = cad + "DESVANECER ";
   }
   return cad;
}

UnicodeString TConexion::cadenaBitsExpresivos(unsigned long vector) {
   UnicodeString cad = "";

   if (vector & EXP_NEUTRAL) {
      cad = cad + "EXP.NEUTRAL ";
   }
   if (vector & EXP_BLINK) {
      cad = cad + "PARPADEAR ";
   }
   if (vector & EXP_WINK_LEFT) {
      cad = cad + "GUIO IZQ. ";
   }
   if (vector & EXP_WINK_RIGHT) {
      cad = cad + "GUIO DER. ";
   }
   if (vector & EXP_HORIEYE) {
      cad = cad + "MIRAR IZQ-DER ";
   }
   if (vector & EXP_EYEBROW) {
      cad = cad + "SUBIR ENTRECEJO ";
   }
   if (vector & EXP_FURROW) {
      cad = cad + "FRUNCIR ENTRECEJO ";
   }
   if (vector & EXP_SMILE) {
      cad = cad + "SONREIR ";
   }
   if (vector & EXP_CLENCH) {
      cad = cad + "APRETAR DIENTES ";
   }
   if (vector & EXP_LAUGH) {
      cad = cad + "REIR ";
   }
   if (vector & EXP_SMIRK_LEFT) {
      cad = cad + "MEJILLA IZQ. ";
   }
   if (vector & EXP_SMIRK_RIGHT) {
      cad = cad + "MEJILLA DER. ";
   }
   return cad;
}


