/***************************************************************
 * Proyecto CoMVeT                                             *
 * Proyecto de fin de carrera de Daniel Hctor Stolfi Rosso    *
 * Escuela Tcnica Superior de Ingeniera Informtica          *
 * Universidad de Mlaga - 2010                                *
 *                                                             *
 * Conexion.h                                                  *
 * Clase abstracta que gestiona las conexiones (cabecera)      *
 * Se crear una instancia de una de sus clases hijas.         *
 *                                                             *
 ***************************************************************/
// ---------------------------------------------------------------------------
#ifndef ConexionH
#define ConexionH
#include <_stddef.h>
#include <system.hpp>
#include "edk.h"
#include "Tipos.h"
// ---------------------------------------------------------------------------
/* Sensibilidad mnima giro */
const int GIRO_UMBRAL = 50;
/* Mximo giro para ponderar el valor de salida */
const int GIRO_MAXIMO = 300;
/* Nmero de seales de electrodos desde el API */
const unsigned int NUM_SENALES_ELECTRODOS = 18;
/* Accin expresiva a utilizar para avanzar */
const EE_ExpressivAlgo_t AVANZAR = EXP_CLENCH;
/* Accin cognitiva a utilizar para retroceder */
const EE_CognitivAction_t RETROCEDER = COG_PULL;

class TConexion {

private:
   EmoEngineEventHandle evento;     // Evento obtenido para operar con el estado
   EmoStateHandle estado;           // Estado actual
   EE_Event_t tipo_evento;          // Tipo de evento
	OptimizationParamHandle optimiz; // Optimizador de eventos cognitivos
   unsigned int usuario;            // Identificador del usuario
   int giroX;                       // Posicin X girscopo
   EE_CognitivAction_t cognitivo;   // Accion cognitiva detectada en el ltimo evento
   EE_ExpressivAlgo_t expresivo;    // Accion expresiva detectada en el ltimo evento
   float potenciaCognitivo;         // Nivel de potencia de la accion cognitiva detectada
   float potenciaExpresivo;         // Nivel de potencia de la accion expresiva detectada
   int cargaBat, maximaBat;         // Nivel de carga y carga mxima de la batera
   EE_SignalStrength_enum wifi;     // Nivel de seal del enlace inalmbrico
   float tiempo;                    // Marca temporal del ltimo evento recibido
   EE_EEG_ContactQuality_t electrodos[NUM_SENALES_ELECTRODOS]; // Vector con el estado de los electrodos

protected:
   int error;                       // Cdigo de error devuelto por la ltima llamada a funcin

public:
   /* Constructor */
   TConexion();

   /* Destructor */
   ~TConexion();

   /* conectar()
      Inicia la conexin. Mtodo abstracto que se implementa en las clases hijas */
   virtual bool conectar() = 0;

   /* desconectar()
      Desconecta del casco o del emulador
      Devuelve true si no se han producido errores */
   bool desconectar();

   /* inicializar()
      Inicializa la conexin activando el optimizador
      Devuelve true si no se han producido errores */
   bool inicializar();

   /* eventoSiguiente()
      Recupera el evento siguiente y devuelve true si ha tenido xito */
   bool eventoSiguiente();

   /* estadoActualizado()
      Devuelve true si se ha actualizado el estado */
   bool estadoActualizado();

   /* dispositivoConectado()
      Devuelve true si se ha conectado el dispositivo */
   bool dispositivoConectado();

   /* dispositivoDesconectado()
      Devuelve true si se ha desconectado el dispositivo */
   bool dispositivoDesconectado();

   /* accion(AC_Accion_t* ac, unsigned int* potencia)
      Devuelve por referencia el tipo de accin del estado actual
      y el nivel de potencia detectado (0 - 10) */
   void accion(AC_Accion_t* ac, unsigned int* potencia);

   /* nivelSenal()
      Devuelve el nivel de seal del enlace wifi */
   SN_Senal_t nivelSenal();

   /* tiempoConexion()
      Devuelve los segundos transcurridos desde la conexin */
   float tiempoConexion();

   /* cargaBateria(int *carga, int *maxima)
      Devuelve por referencia la carga de la bateria y el nivel mximo de carga */
   void cargaBateria(int* carga, int* maxima);

   /* estadoElectrodos(int* contactos, bool* ok)
      Devuelve por referencia el array de electrodos con la calidad de contacto de cada uno
      y si el estado es suficiente para una deteccin de calidad */
   void estadoElectrodos(int* contactos, bool* ok);

   /* nivelEntrenamiento()
      Devuelve el nivel de entrenamiento del perfil de usuario activo (0.0 - 1.0) */
   float nivelEntrenamiento();

   /* cadenaEstado()
      Devuelve en una cadena el contenido del ltimo estado detectado */
   UnicodeString cadenaEstado();

   /* cadenaPerfil()
      Devuelve una cadena descriptiva del perfil de usuario activo */
   UnicodeString cadenaPerfil();

   /* cargaPerfil(char* fichero)
      Carga desde el fichero el perfil de usuario
      Devuelve true si no se han producido errores */
   bool cargaPerfil(char* fichero);

   /* setSensibilidadAvance(unsigned int nivel)
      Cambia la sensibilidad para la accin de avance (0 - 1000) en el perfil de usuario activo
      Devuelve true si no se han producido errores */
   bool setSensibilidadAvance(unsigned int nivel);

   /* getSensibilidadAvance()
      Devuelve la sensibilidad para la accin de avance (0 - 1000) del perfil de usuario activo */
   unsigned int getSensibilidadAvance();

   /* setSensibilidadRetroceso(unsigned int nivel)
      Cambia la sensibilidad para la accin de retroceso (1 - 9) en el perfil de usuario activo
      Devuelve true si no se han producido errores */
   bool setSensibilidadRetroceso(unsigned int nivel);

   /* getSensibilidadRetroceso()
      Devuelve la sensibilidad para la accin de retroceso (1 - 9) del perfil de usuario activo */
   unsigned int getSensibilidadRetroceso();

   /* giro(GI_Giro_t* gi)
      Devuelve la accin detectada desde el acelermetro */
   void giro(GI_Giro_t* gi);

   /* aceleracionGiro()
      Devuelve el valor diferencial del movimiento detectado */
   unsigned int aceleracionGiro();

   /* inicializaGiro()
      Inicializa el acelermetro en la posicin actual del casco */
   bool inicializaGiro();

   /* tipoEstado()
      Devuelve una cadena descriptiva del ltimo estado detectado */
   UnicodeString tipoEstado();

   /* cadenaOptimizacion()
      Devuelve una cadena con la informacin de las optimizaciones activas */
   UnicodeString cadenaOptimizacion();

   /* cadenaError()
      Devuelve una cadena con la descripcin del ltimo error acontecido */
   UnicodeString cadenaError();

private:
   /* cadenaBitsCognitivos(unsigned long vector)
      Convierte los bits cognitivos utilizados por el programa que se obtienen
      desde el vector pasado como parmetro en una cadena de caracteres */
   UnicodeString cadenaBitsCognitivos(unsigned long vector);

   /* cadenaBitsExpresivos(unsigned long vector)
      Convierte los bits expresivos utilizados por el programa que se obtienen
      desde el vector pasado como parmetro en una cadena de caracteres */
   UnicodeString cadenaBitsExpresivos(unsigned long vector);

};
#endif
